/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package calc;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JToggleButton;

import common.FileIO;
import common.SelectTextItemListener;


public class CalculatorView<T extends Number> extends JFrame {

	private final CalculatorVisualizer<T> visualizer;
	private boolean calculating;
	private JToggleButton helpButton;
	public CalculatorView(CalculatorVisualizer<T> vis) {
		super("Calculator");
		this.visualizer = vis;
		initGui();
	}
	public CalculatorVisualizer<T> getVisualizer() {
		return visualizer;
	}
	private void initGui() {
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		getContentPane().setLayout(gridbag);
		c.insets = new Insets(10,5,5,5);
		c.fill = GridBagConstraints.BOTH;
		c.weightx = 1.0;
		c.weighty = 1.0;
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.anchor = GridBagConstraints.CENTER;
		gridbag.setConstraints(visualizer,c);
		getContentPane().add(visualizer);
		c.insets = new Insets(0,0,0,0);
		c.fill = GridBagConstraints.BOTH;
		c.weighty = 0;
		c.gridy = 1;
		c.anchor = GridBagConstraints.EAST;
		JPanel p = createButtonPanel();
		gridbag.setConstraints(p,c);
		getContentPane().add(p);
		pack();
		setSize(500,300);
		setVisible(true);
	}
	private JPanel createButtonPanel() {
		JPanel panel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		helpButton = new JToggleButton("Help");
		String text = new FileIO<T>(null).loadInfoText("Calculator");
		String title = "Calculator Info";
		helpButton.addItemListener(new SelectTextItemListener(helpButton,title,text,620,600));
		panel.add(helpButton);
		JButton b = new JButton("Remove Row");
		b.addMouseListener(new MouseAdapter() {

			@Override
			public void mouseReleased(MouseEvent e) {
				JTable table = visualizer.getCalculatorTable();
				int rIdx = table.getEditingRow();
				if(rIdx!=-1) {
					visualizer.stopEditing();
					visualizer.getModel().removeRow(rIdx);
				}
			}
		});
		panel.add(b);
		b = new JButton("Insert Row");
		b.addMouseListener(new MouseAdapter() {

			@Override
			public void mouseReleased(MouseEvent e) {
				JTable table = visualizer.getCalculatorTable();
				int rIdx = table.getEditingRow();
				if(rIdx!=-1) {
					visualizer.stopEditing();
					visualizer.getModel().insertRow(rIdx);
				}
			}
		});
		panel.add(b);
		b = new JButton("Calculate");
		b.addMouseListener(new MouseAdapter() {

			@Override
			public void mouseReleased(MouseEvent ev) {
				if(!calculating) {
					calculating = true;
					Thread t = new Thread() {

						public void run() {
							try {
								visualizer.stopEditing();
								visualizer.getModel().calculate();
							} catch(Exception e) {
								JOptionPane.showMessageDialog(CalculatorView.this,e.getMessage());
							}
							calculating = false;
						}
					};
					t.setPriority(Thread.MIN_PRIORITY);
					t.start();
				}
			}
		});
		panel.add(b);
		return panel;
	}
	@Override
	public void dispose(){
		helpButton.setSelected(false);
		super.dispose();
	}
}
