package common;
/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/

import java.awt.Color;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Point;
import java.io.ByteArrayOutputStream;
import java.io.PrintStream;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;

@SuppressWarnings("serial")
public class Log extends JPanel {

	private final static int BUFFER_SIZE = 20000;
	private final StringBuffer buf;
	private static Log instance;
	private boolean enable = true;
	private UpdateTextThread update;
	private JTextPane textPane;
	private JScrollPane scrollPane;
	private Log() {
		buf = new StringBuffer();
	}
	public static Log getIstc() {
		if(instance==null)
			instance = new Log();
		return instance;
	}
	public void initGUI(){
		if(textPane==null){
			GridBagLayout gbl = new GridBagLayout();
			GridBagConstraints gbc = new GridBagConstraints();
			setLayout(gbl);
			gbc.fill = GridBagConstraints.BOTH;
			gbc.gridx = 0;
			gbc.gridy = 0;
			gbc.weightx = 1;
			gbc.weighty = 1;
			textPane = new JTextPane();
			textPane.setEditable(false);
			textPane.setFont(new Font(Font.MONOSPACED,Font.PLAIN,9));
			textPane.setBackground(new Color(255,255,220));
			scrollPane = new JScrollPane(textPane);
			gbl.setConstraints(scrollPane,gbc);
			add(scrollPane);
			log("");
		}
	}
	public synchronized void log(String s) {
		if(!enable)
			return;
		buf.append(s);
		if(textPane!=null&&update==null) {
			update = new UpdateTextThread();
			update.start();
		}
		System.out.print(s);
	}
	public synchronized void logln(String s) {
		log(s);
		log("\n");
	}
	public synchronized void logf(String s,int width) {
		log(s);
		for(int i = s.length();i<width;i++)
			log(" ");
	}
	public void log(Exception e) {
		logln(e.toString());
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		PrintStream ps = new PrintStream(baos);
		e.printStackTrace(ps);
		logln(new String(baos.toByteArray()));
	}
	private class UpdateTextThread extends Thread {

		@Override
		public void run() {
			try {
				Thread.sleep(500);
			} catch(InterruptedException e) {}
			synchronized(Log.this) {
				Point pos = scrollPane.getViewport().getViewPosition();
				if(buf.length()>BUFFER_SIZE)
					buf.delete(0,buf.length()-BUFFER_SIZE);
				textPane.setText(buf.toString());
				pos.y = textPane.getHeight();
				scrollPane.getViewport().setViewPosition(pos);
				update = null;
			}
		}
	}
	public void enableLogging(boolean b) {
		enable  = b;
	}
	public void deleteGUI() {
		textPane = null;
		scrollPane = null;
		removeAll();
	}
}
