/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package common;
import java.util.Arrays;
import java.util.HashMap;


public final class Profiler {

	private final HashMap<String,Long> sumTime;
	private final HashMap<String,Long> iniTime;
	private final HashMap<String,Long> calls;
	private static Profiler inst;
	private long time;
	private boolean active;
	private Profiler() {
		sumTime = new HashMap<String,Long>();
		iniTime = new HashMap<String,Long>();
		calls = new HashMap<String,Long>();
	}
	public static Profiler getInstance() {
		if(inst==null)
			inst = new Profiler();
		return inst;
	}
	public void startTimeMeas(String method) {
		if(active)
			if(iniTime.get(method)==null)
				iniTime.put(method,System.currentTimeMillis());
			else
				Log.getIstc().logln("Profiler Error: Initialized running measurement");
	}
	public void endTimeMeas(String method) {
		if(active) {
			Long startTime = iniTime.get(method);
			if(startTime!=null) {
				Long t = sumTime.get(method);
				if(t==null)
					t = (long)0;
				sumTime.put(method,t+System.currentTimeMillis()-startTime);
				iniTime.put(method,null);
				Long c = calls.get(method);
				if(c==null)
					c = (long)0;
				calls.put(method,++c);
			} else
				Log.getIstc().logln("Profiler Error: Stopping non running measurement");
		}
	}
	public String toString() {
		if(active) {
			StringBuffer sb = new StringBuffer();
			sb.append("method : calls time[s] time[%]\n");
			long runTime = System.currentTimeMillis()-time;
			String[] keys = sumTime.keySet().toArray(new String[sumTime.size()]);
			Arrays.sort(keys);
			for(String method:keys) {
				float mt = sumTime.get(method)/1000f;
				float p = sumTime.get(method)*100f/runTime;
				long c = calls.get(method);
				sb.append(method+": "+c+" "+mt+"s "+p+"%\n");
			}
			sb.append("total: "+runTime/1000f);
			return sb.toString();
		}
		return null;
	}
	public boolean isActive() {
		return active;
	}
	public void setActive(boolean active) {
		this.active = active;
		if(!active) {
			iniTime.clear();
			sumTime.clear();
		} else
			time = System.currentTimeMillis();
	}
}
