/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package common;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.AbstractButton;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingUtilities;

public final class SelectTextItemListener implements ItemListener,ActionListener {

	private JFrame frame;
	private final String text;
	private final String title;
	private final AbstractButton item;
	private final int width;
	private final int height;
	public SelectTextItemListener(AbstractButton ab,String title,String text,int w,int h) {
		this.text = text;
		this.title = title;
		this.item = ab;
		this.width = w;
		this.height = h;
	}
	@Override
	public void itemStateChanged(ItemEvent e) {
		handleFrame();
	}
	@Override
	public void actionPerformed(ActionEvent arg0) {
		handleFrame();
	}
	private void handleFrame() {
		if(frame==null) {
			frame = new JFrame(title);
			frame.setBackground(Color.black);
			frame.addWindowListener(new WindowAdapter() {

				@Override
				public void windowClosed(WindowEvent e) {
					item.setSelected(false);
					frame.removeWindowListener(this);
					frame = null;
				}
				@Override
				public void windowClosing(WindowEvent e) {
					item.setSelected(false);
					frame.removeWindowListener(this);
					frame = null;
				}
			});
			//JTextArea textArea = new JTextArea(text);
			JEditorPane textArea = new JEditorPane("text/html",text);
			//textArea.setLineWrap(true);
			textArea.setEditable(false);
			//Font font = new Font(Font.MONOSPACED,Font.BOLD,12);
			//textArea.setFont(font);
			//textArea.setBackground(Color.BLACK);
			//textArea.setForeground(Color.WHITE);
			final JScrollPane scrollPane = new JScrollPane(textArea);
			int policy = ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER;
			scrollPane.setHorizontalScrollBarPolicy(policy);
			frame.getContentPane().add(scrollPane);
			frame.pack();
			frame.setMinimumSize(new Dimension(width,height));
			frame.setMaximumSize(new Dimension(width,height));
			frame.setSize(width,height);
			frame.setVisible(true);
			SwingUtilities.invokeLater(new Runnable() {
				
				@Override
				public void run() {
					Point pos = scrollPane.getViewport().getViewPosition();
					pos.y = 0;
					scrollPane.getViewport().setViewPosition(pos);					
				}
			});
		} else
			frame.dispose();
	}
}
