/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package math;
import java.math.BigDecimal;
import java.util.List;

public abstract class Util<T extends Number> {

	public static final int MAX_INT = Integer.MAX_VALUE;
	public static final int MIN_INT = Integer.MIN_VALUE+1;
	public Util() {}
	public abstract T getPi(int digits);
	public abstract T getE(int digits);
	public abstract T cast(int i);
	public abstract T cast(float f);
	public abstract T cast(double d);
	public final T[] cast(Float[] n) {
		T[] ret = array(n.length);
		for(int i = 0;i<ret.length;i++)
			ret[i] = cast(n[i]);
		return ret;
	}
	public final T[] cast(Double[] n) {
		T[] ret = array(n.length);
		for(int i = 0;i<ret.length;i++)
			ret[i] = cast(n[i]);
		return ret;
	}
	public final T[] cast(float[] n) {
		T[] ret = array(n.length);
		for(int i = 0;i<ret.length;i++)
			ret[i] = cast(n[i]);
		return ret;
	}
	public final T[] cast(double[] n) {
		T[] ret = array(n.length);
		for(int i = 0;i<ret.length;i++)
			ret[i] = cast(n[i]);
		return ret;
	}
	public final Float[] cast2FloatArray(T[] n) {
		Float[] ret = new Float[n.length];
		for(int i = 0;i<ret.length;i++)
			ret[i] = n[i].floatValue();
		return ret;
	}
	public final Double[] cast2DoubleArray(T[] n) {
		Double[] ret = new Double[n.length];
		for(int i = 0;i<ret.length;i++)
			ret[i] = n[i].doubleValue();
		return ret;
	}
	public final float[] cast2floatArray(T[] n) {
		float[] ret = new float[n.length];
		for(int i = 0;i<ret.length;i++)
			ret[i] = n[i].floatValue();
		return ret;
	}
	public final double[] cast2doubleArray(T[] n) {
		double[] ret = new double[n.length];
		for(int i = 0;i<ret.length;i++)
			ret[i] = n[i].doubleValue();
		return ret;
	}
	public final float[] cast2floatArray(List<T[]> l) {
		float[] ret = new float[l.size()*2];
		for(int i = 0;i<l.size();i++) {
			ret[2*i] = l.get(i)[0].floatValue();
			ret[2*i+1] = l.get(i)[1].floatValue();
		}
		return ret;
	}
	public final int[] cast2intArray(T[] n) {
		int[] ret = new int[n.length];
		for(int i = 0;i<ret.length;i++)
			ret[i] = n[i].intValue();
		return ret;
	}
	public abstract T parse(String s) throws NumberFormatException;
	public abstract T copy(T x,int p);
	public abstract T round(T x,int p);
	public abstract T expand(T x,int p);
	public final void round(T[] arr,int p) {
		for(int i = 0;i<arr.length;i++)
			arr[i] = round(arr[i],p);
	}
	public final void setPrecision(T[] arr,int p) {
		for(int i = 0;i<arr.length;i++)
			arr[i] = copy(arr[i],p);
	}
	public abstract int precision(T x);
	public final int minPrecision(T[] arr) {
		int retVal = precision(arr[0]);
		for(int i = 1;i<arr.length;i++) {
			int p = precision(arr[i]);
			if(p<retVal)
				retVal = p;
		}
		return retVal;
	}
	public final int maxPrecision(T[] arr) {
		int retVal = precision(arr[0]);
		for(int i = 1;i<arr.length;i++) {
			int p = precision(arr[i]);
			if(p>retVal)
				retVal = p;
		}
		return retVal;
	}
	public final T truncate(T x,int ac) {
		int sc = getScale(x);
		if(sc!=MIN_INT&&sc+ac<precision(x))
			x = copy(x,sc+ac+1);
		return x;
	}
	public final void copy(T[] v1,T[] v2) {
		for(int i = 0;i<v2.length;i++)
			v2[i] = v1[i];
	}
	public abstract int getScale(T x);
	public abstract int getErrorScale(T x);
	public abstract T[] array(int l);
	public abstract T[][] array(int l1,int l2);
	public static final int maxValue(int[][] vals) {
		int retVal = Integer.MIN_VALUE;
		for(int i = 0;i<vals.length;i++) {
			for(int j = 0;j<vals[i].length;j++)
				if(retVal<vals[i][j])
					retVal = vals[i][j];
		}
		return retVal;
	}
	public static final float maxValue(float[][] vals) {
		float retVal = Float.MIN_VALUE;
		for(int i = 0;i<vals.length;i++) {
			for(int j = 0;j<vals[i].length;j++)
				if(retVal<vals[i][j])
					retVal = vals[i][j];
		}
		return retVal;
	}
	public String toPlainString(T t) {
		if(t!=null)
			return new BigDecimal(t.toString()).toPlainString();
		return null;
	}
}
