/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package math;

import org.apfloat.Apcomplex;
import org.apfloat.Apfloat;
import org.apfloat.ApfloatMath;
import org.apfloat.Aprational;


public final class UtilApfloat extends Util<Apfloat> {

	private final static Apfloat HALF = new Apfloat("0.5").precision(Apcomplex.INFINITE);
	public UtilApfloat() {}
	@Override
	public Apfloat cast(int i) {
		return new Apfloat(i);
	}
	@Override
	public Apfloat cast(float f) {
		return new Apfloat(f);
	}
	@Override
	public Apfloat cast(double d) {
		return new Apfloat(d);
	}
	@Override
	public Apfloat copy(Apfloat x,int p) {
		return x.precision(p);
	}
	@Override
	public Apfloat round(Apfloat x,int p) {
		int scale = getScale(x);
		if(scale+1+x.precision()<p)
			return x;
		Apfloat ret = ApfloatMath.scale(ApfloatMath.scale(x,p).add(HALF).floor(),-p);
		if(scale>-1)
			p += scale+1;
		return ret.precision()>p?ret.precision(p):ret;
	}
	@Override
	public Apfloat expand(Apfloat x,int p) {
		return new Apfloat(x.toString(),p);
	}
	@Override
	public int precision(Apfloat x) {
		long p = x.precision();
		if(p==Apcomplex.INFINITE||p>MAX_INT)
			return MAX_INT;
		return (int)p;
	}
	@Override
	public Apfloat parse(String s) throws NumberFormatException {
		if(s.indexOf("/")!=-1)
			return new Aprational(s);
		return new Apfloat(s);
	}
	@Override
	public Apfloat getPi(int digits) {
		return ApfloatMath.pi(digits);
	}
	@Override
	public Apfloat getE(int digits) {
		return ApfloatMath.exp(copy(Apcomplex.ONE,digits));
	}
	@Override
	public Apfloat[] array(int dim) {
		return new Apfloat[dim];
	}
	@Override
	public Apfloat[][] array(int d1,int d2) {
		return new Apfloat[d1][d2];
	}
	@Override
	public int getScale(Apfloat x) {
		long s = x.scale();
		if(s==-Apcomplex.INFINITE||s<MIN_INT)
			return MIN_INT;
		return (int)s-1;
	}
	@Override
	public int getErrorScale(Apfloat x) {
		int s = getScale(x);
		if(s==MIN_INT)
			return s;
		int p = precision(x);
		if(p==MAX_INT)
			return MIN_INT;
		return s-precision(x)+1;
	}
	@Override
	public String toPlainString(Apfloat t) {
		if(t instanceof Aprational) {
			String strg = t.toString();
			int idx = strg.indexOf("/");
			Apfloat x = new Apfloat(strg.substring(0,idx));
			Apfloat y = new Apfloat(strg.substring(idx+1));
			t = x.divide(y);
		}
		return super.toPlainString(t);
	}
}
