/**
	Copyright (c) 2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package mvc;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Insets;

import javax.swing.JPanel;

public class CompoundBorder implements IFZLBorder {
	IFZLBorder outer;
	IFZLBorder inner;
	JPanel panel;	
	public CompoundBorder(IFZLBorder b1,IFZLBorder b2){
		outer = b1;
		inner = b2;
		panel = new JPanel();
	}
	@Override
	public void drawBorder(Component c,Graphics g) {
		outer.drawBorder(c,g);
		Insets o = outer.getBorderInsets(c);
		int w = c.getWidth()-o.left-o.right;
		int h = c.getHeight()-o.top-o.bottom;
		panel.setSize(w,h);
		g.translate(o.left,o.top);
		inner.drawBorder(panel,g);
		g.translate(-o.left,-o.top);
	}
	@Override
	public Insets getBorderInsets(Component c) {
		Insets o = outer.getBorderInsets(c);
		Insets i = inner.getBorderInsets(c);
		int t = i.top+o.top;
		int r = i.right+o.right;
		int b = i.bottom+o.bottom;
		int l = i.left+o.left;
		return new Insets(t,r,b,l);
	}
	@Override
	public void setColor(Color c) {
		outer.setColor(c);
		inner.setColor(c);
	}
}
