/**
	Copyright (c) 2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package mvc;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

import javax.swing.ImageIcon;

import math.VecMath;
import pattern.LightChar;

public class LightCharBorder<T extends Number> extends LightCharMap<T> 
implements IFZLBorder {
	private final float SHRINK = 0.6f;

	public LightCharBorder(LightChar[] lcs,VecMath<T> mt,VecMath<Float> fmt) {
		super(lcs,mt,fmt);
	}
	@Override
	public Insets getBorderInsets(Component c) {
		float fSize = (c.getWidth()+c.getHeight())/(lChars.length/2+2);
		int rows = Math.round(c.getHeight()/fSize);
		int cols = Math.round(c.getWidth()/fSize);
		if(rows+cols-2>lChars.length/2)
			if(rows>cols)
				rows--;
			else
				cols--;
		int rSize = (int)(2.3*SHRINK*c.getHeight()*1.0f/rows);
		int cSize = (int)(2.3*SHRINK*c.getWidth()*1.0f/cols);
		return new Insets(rSize,cSize,rSize,cSize);
	}
	public void drawBorder(final Component c,Graphics g){
		if(refreshComp==null)
			refreshComp = c;
		float fSize = (c.getWidth()+c.getHeight())/(lChars.length/2+2);
		int iSize = (int) (Math.round(fSize)*0.8f);
		if(iconSize!=iSize||iconMap.size()!=lChars.length){
			buildIcons(iSize);
			iconSize = iSize;
		}
		int rows = Math.round(c.getHeight()/fSize);
		int cols = Math.round(c.getWidth()/fSize);
		if(rows+cols-2>lChars.length/2)
			if(rows>cols)
				rows--;
			else
				cols--;
		float rSize = c.getHeight()*1.0f/rows;
		float cSize = c.getWidth()*1.0f/cols;
		int dy = (int) (2.3*SHRINK*rSize);
		int dx = (int) (2.3*SHRINK*cSize);
		g.setColor(new Color(60,60,60));
		g.fillRect(0,0,c.getWidth(),dy);
		g.fillRect(0,c.getHeight()-dy,c.getWidth(),dy);
		g.fillRect(0,dy,dx,c.getHeight()-2*dy);
		g.fillRect(c.getWidth()-dx,dy,dx,c.getHeight()-2*dy);
		float cGap = (cSize-iconSize)/2;
		float rGap = (rSize-iconSize)/2; 
		ArrayList<int[]> cells = new ArrayList<int[]>(); 
		for(int col=0;col<cols;col++)
			cells.add(new int[]{col,0});
		for(int row=1;row<rows;row++)
			cells.add(new int[]{cols-1,row});
		for(int col=cols-2;col>=0;col--)
			cells.add(new int[]{col,rows-1});
		for(int row=rows-2;row>=0;row--)
			cells.add(new int[]{0,row});

		int idx = 0;
		for(int i = 0;i<cells.size();i++){
			int col = cells.get(i)[0];
			int row = cells.get(i)[1];
			if(idx>=lChars.length)
				continue;
			LightChar lc = lChars[idx++];
			if(iconMap.get(lc.getChar())==null)
				continue;
			ImageIcon icon = iconMap.get(lc.getChar());
			icon.paintIcon(c,g,(int)(col*cSize+cGap),(int)(row*rSize+rGap));
		}
		float rSize2 = (c.getHeight()-2*SHRINK*rSize)/rows;
		float cSize2 = (c.getWidth()-2*SHRINK*cSize)/cols;
		FontRenderContext rctx = ((Graphics2D) g).getFontRenderContext();
		Font font = g.getFont();
		int fntsz = (int) (font.getSize()*0.05f*iconSize);
		g.setFont(new Font(font.getFontName(),font.getStyle(),fntsz));
		Rectangle2D bnd = g.getFont().getStringBounds("Q",rctx);
		int bs = (int)((bnd.getHeight()+bnd.getWidth())*0.9f);
		idx = 0;
		Color dcol = g.getColor().darker();
		Color bcol = g.getColor().brighter().brighter();
		bcol = new Color(bcol.getRed(),bcol.getGreen(),bcol.getBlue(),130);
		for(int i = 0;i<cells.size();i++){
			int col = cells.get(i)[0];
			int row = cells.get(i)[1];
			if(idx<lChars.length){
				LightChar lc = lChars[idx++];
				int x1 = (int)((col+0.5)*cSize);
				int y1 = (int)((row+0.5)*rSize);
				int x2 = (int)((col+0.5)*cSize2+SHRINK*cSize);
				int y2 = (int)((row+0.5)*rSize2+SHRINK*rSize);
				g.setColor(bcol);
				g.fillOval(x2-bs/2,y2-bs/2,bs,bs);
				int vx = x2-x1;
				int vy = y2-y1;
				x1 = (int)(x1+0.4*vx);
				y1 = (int)(y1+0.4*vy);
				x2 = (int)(x2-0.2*vx);
				y2 = (int)(y2-0.2*vy);
				g.drawLine(x1,y1,x2,y2);
				g.setColor(dcol);
				bnd = g.getFont().getStringBounds(lc.getChar(),rctx);
				float cGap2 = (float) ((cSize2-bnd.getWidth())/2);
				float rGap2 = (float) ((rSize2-bnd.getHeight())/2);
				int sx = (int)(col*cSize2+cGap2+SHRINK*cSize);
				int sy = (int)(row*rSize2+rGap2+SHRINK*rSize);
				int sy2 = (int)(sy+bnd.getHeight()*3/4f+2);
				g.drawString(lc.getChar(),sx,sy2);
			}
		}
	}
	@Override
	public void setColor(Color c) {}
}
