/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package mvc;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;

import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.border.MatteBorder;

import math.VecMath;
import pattern.LightChar;

@SuppressWarnings("serial")
public class LightCharTable<T extends Number> extends JPanel {

	private final LightCharMap<T> map;
	public LightCharTable(LightChar[] lcs,VecMath<T> mt,VecMath<Float> fmt) {
		this.map = new LightCharMap<T>(lcs,mt,fmt);
		setBackground(new Color(50,50,50));
		Color c = getBackground().brighter().brighter();
		this.setBorder(new MatteBorder(10,10,10,10,c));
	}
	@Override
	public void paintComponent(Graphics g) {
		if(map.refreshComp==null)
			map.refreshComp = this;
		int w = getWidth()-20;
		int h = getHeight()-20;
		float fSize = (float) Math.sqrt((w*h/(2*map.lChars.length)));
		int rows = Math.round(h/fSize);
		int cols = Math.round(w/fSize);
		int iSize = (int) (Math.round(fSize)*0.8f);
		if(cols%2==1){
			cols--;
			rows++;
			iSize = (int) (Math.round(fSize)*0.7f);
		} 
		if(map.iconSize!=iSize||map.iconMap.size()!=map.lChars.length){
			map.buildIcons(iSize);
			map.iconSize = iSize; 
		}
		float rSize = h*1.0f/rows;
		float cSize = w*1.0f/cols;
		g.translate(10,10);
		g.setColor(getBackground());
		g.fillRect(0,0,w,h);
		float cGap = (cSize-map.iconSize)/2;
		float rGap = (rSize-map.iconSize)/2;
		int idx = 0;
		for(int col=0;col<cols;col+=2)
			for(int row=0;row<rows;row++){
				if(idx<map.lChars.length){
					LightChar lc = map.lChars[idx++];
					ImageIcon icon = map.iconMap.get(lc.getChar());
					if(icon!=null)
						icon.paintIcon(this,g,(int)(col*cSize+cGap),(int)(row*rSize+rGap));
				}
			}
		FontRenderContext rctx = ((Graphics2D) g).getFontRenderContext();
		Font font = g.getFont();
		int fntsz = font.getSize()*map.iconSize/LightCharMap.ICON_INI_SIZE;
		g.setFont(new Font(font.getFontName(),font.getStyle(),fntsz));
		Rectangle2D bnd = g.getFont().getStringBounds("Q",rctx);
		int bs = (int)((bnd.getHeight()+bnd.getWidth())*0.9f);
		idx = 0;
		Color dcol = g.getColor().darker();
		Color bcol = g.getColor().brighter().brighter();
		for(int col=1;col<cols;col+=2)
			for(int row=0;row<rows;row++){
				if(idx<map.lChars.length){
					LightChar lc = map.lChars[idx++];
					int x1 = (int)(((col-1)+0.5)*cSize);
					int y1 = (int)((row+0.5)*rSize);
					int x2 = (int)((col+0.5)*cSize);
					int y2 = (int)((row+0.5)*rSize);
					g.setColor(bcol);
					g.fillOval(x2-bs/2,y2-bs/2,bs,bs);
					int vx = x2-x1;
					int vy = y2-y1;
					x1 = (int)(x1+0.6*vx);
					y1 = (int)(y1+0.6*vy);
					x2 = (int)(x2-0.2*vx);
					y2 = (int)(y2-0.2*vy);
					g.drawLine(x1,y1,x2,y2);
					g.setColor(dcol);
					bnd = g.getFont().getStringBounds(lc.getChar(),rctx);
					float cGap2 = (float) ((cSize-bnd.getWidth())/2);
					float rGap2 = (float) ((rSize-bnd.getHeight())/2);
					int sx = (int)(col*cSize+cGap2);
					int sy = (int)(row*rSize+rGap2);
					int sy2 = (int)(sy+bnd.getHeight()*3/4f+2);
					g.drawString(lc.getChar(),sx,sy2);
				}
			}
		g.translate(-10,-10);
	}
	public void dispose(){
		map.dispose();
	}
}
