/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package mvc.scan;

public final class Observables {
	
	public final static byte OBS_REFL_CNT = 0;
	public final static byte OBS_LENGTH = 1;
	public final static byte OBS_OBS1 = 2;

	public int[][] reflCnt;
	public float[][] length;
	public float[][] obs1;
	public int maxReflCnt = 1;
	public float maxLength = 1;
	public float maxObs1 = 1;
	public boolean maxReflCntChanged;
	public boolean maxLengthChanged;
	public boolean maxObs1Changed;
	public int aproxPixels = 0;
	public Observables(int width,int height,Observables obs) {
		reflCnt = new int[width][height];
		length = new float[width][height];
		obs1 = new float[width][height];
		if(obs!=null) {
			maxReflCnt = obs.maxReflCnt;
			maxLength = obs.maxLength;
			maxObs1 = obs.maxObs1;
		}
	}
	public void scaledCopy(Observables obs) {
		float invWidth = 1f/reflCnt.length;
		float invHeight = 1f/reflCnt[0].length;
		for(int w = 0;w<reflCnt.length;w++) {
			for(int h = 0;h<reflCnt[0].length;h++) {
				int wo = (int)(w*invWidth*obs.reflCnt.length);
				int ho = (int)(h*invHeight*obs.reflCnt[0].length);
				reflCnt[w][h] = obs.reflCnt[wo][ho];
				length[w][h] = obs.length[wo][ho];
				obs1[w][h] = obs.obs1[wo][ho];
			}
		}
	}
	public Number getMax(byte selObs) {
		switch(selObs) {
			case OBS_REFL_CNT:
				return maxReflCnt;
			case OBS_LENGTH:
				return maxLength;
			case OBS_OBS1:
				return maxObs1;
		}
		return null;
	}
	public boolean isMaxChanged(byte selObs) {
		switch(selObs) {
			case OBS_REFL_CNT:
				return maxReflCntChanged;
			case OBS_LENGTH:
				return maxLengthChanged;
			case OBS_OBS1:
				return maxObs1Changed;
		}
		return false;
	}
	public void setMaxChanged(byte selObs,boolean b) {
		switch(selObs) {
			case OBS_REFL_CNT:
				maxReflCntChanged = b;
				break;
			case OBS_LENGTH:
				maxLengthChanged = b;
				break;
			case OBS_OBS1:
				maxObs1Changed = b;
				break;
		}
	}
}
