/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package mvc.var;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import javax.swing.JTextField;

import math.VecMath;
import mvc.Controller;

public abstract class BaseKeyListener<T extends Number> extends KeyAdapter {

	static final char LIMIT_CHAR = '&';
	protected final Controller<T> ctrl;
	private final boolean autoExpand;
	public BaseKeyListener(Controller<T> c,boolean exp) {
		this.ctrl = c;
		this.autoExpand = exp;
	}
	@Override
	public abstract void keyReleased(KeyEvent e);
	protected static boolean removeNonNumberChars(JTextField tf) {
		String valTxt1 = tf.getText();
		StringBuffer valTxt2 = new StringBuffer(valTxt1.trim());
		boolean expand = true;
		for(int i = 0;i<valTxt2.length();i++) {
			char c = valTxt2.charAt(i);
			if(c==LIMIT_CHAR)
				expand = false;
			if(c!='.'&&c!='-'&&!Character.isDigit(c))
				valTxt2.deleteCharAt(i--);
		}
		if(valTxt1.length()!=valTxt2.length())
			tf.setText(valTxt2.toString());
		return expand;
	}
	protected T getValue(String text) {
		return getValue(text,autoExpand);
	}
	protected T getValue(String text,boolean exp) {
		StringBuffer valTxt = new StringBuffer(text);
		if(valTxt.indexOf(".")==-1)
			valTxt.append(".0");
		VecMath<T> mt = ctrl.getModel().getMath();
		if(autoExpand||exp) {
			T retVal = mt.util.parse(valTxt.toString());
			for(int pr = mt.util.precision(retVal);pr<=mt.PRECISION;pr++)
				valTxt.append("0");
		}
		T val = mt.util.parse(valTxt.toString());
		if(autoExpand||exp)
			val = mt.util.copy(val,mt.PRECISION);
		return val;
	}
}