/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package mvc.var;
import java.awt.event.KeyEvent;

import javax.swing.JTextField;

import math.VecMath;
import mvc.Controller;
import mvc.scan.ScanVisualizer;

import common.Log;

class GroupedScanKeyListener<T extends Number> extends BaseKeyListener<T> {

	private final JTextField[] textFields;
	GroupedScanKeyListener(Controller<T> c,JTextField phiMin,JTextField phiMax,JTextField hMin,
	        JTextField hMax) {
		super(c,true);
		this.textFields = new JTextField[]{phiMin,phiMax,hMin,hMax};
	}
	@Override
	public void keyReleased(KeyEvent e) {
		if(e.getKeyCode()==KeyEvent.VK_ENTER) {
			ScanVisualizer<T> scanVis = ctrl.getView().getScanVisualizer();
			scanVis.stopScan("Stopped on zoom frame change");
			VecMath<T> mt = ctrl.getModel().getMath();
			T[] values = mt.util.array(textFields.length);
			try {
				for(int i = 0;i<textFields.length;i++) {
					removeNonNumberChars(textFields[i]);
					values[i] = getValue(textFields[i].getText());
				}
				T[] p1 = mt.util.array(2);
				p1[0] = mt.mul(values[0],mt.PI);
				p1[1] = values[2];
				T[] p2 = mt.util.array(2);
				p2[0] = mt.mul(values[1],mt.PI);
				p2[1] = values[3];
				String msg = "Frame area is zero";
				if(mt.cmp(p1[0],p2[0])!=0&&mt.cmp(p1[1],p2[1])!=0) {
					scanVis.prepareScan(p1,p2);
					scanVis.startScan();
					msg = "Scan started";
				}
				ctrl.getView().getVariableVisualizer().setMessage(msg);
			} catch(NumberFormatException ex) {
				Log.getIstc().log(ex);
			}
		}
	}
}