/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package mvc.var;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JRadioButtonMenuItem;

import common.FileIO;
import common.SelectTextItemListener;


@SuppressWarnings("serial")
public class VariableMenuBar<T extends Number> extends JMenuBar {

	private final VariableVisualizer<T> vis;
	private JCheckBoxMenuItem helpItem;
	public VariableMenuBar(VariableVisualizer<T> vis) {
		this.vis = vis;
		init();
	}
	private void init() {
		JMenu optMenu = new JMenu("Options");
		JRadioButtonMenuItem oItem = new JRadioButtonMenuItem("Truncate");
		oItem.addItemListener(new ItemListener() {

			@Override
			public void itemStateChanged(ItemEvent e) {
				if(e.getStateChange()==ItemEvent.SELECTED)
					vis.enableTruncation(true);
				else
					vis.enableTruncation(false);
			}
		});
		oItem.setSelected(true);
		oItem.setSelected(true);
		optMenu.add(oItem);
		JRadioButtonMenuItem errorItem = new JRadioButtonMenuItem("Show Error");
		errorItem.addItemListener(new ItemListener() {

			@Override
			public void itemStateChanged(ItemEvent e) {
				if(e.getStateChange()==ItemEvent.SELECTED)
					vis.enableErrorMsg(true);
				else
					vis.enableErrorMsg(false);
			}
		});
		optMenu.add(errorItem);
		JMenu infoMenu = new JMenu("?");
		helpItem = new JCheckBoxMenuItem("help...");
		String txt = new FileIO<T>(null).loadInfoText("Variables");
		String title = "Variables Info";
		helpItem.addItemListener(new SelectTextItemListener(helpItem,title,txt,550,500));
		infoMenu.add(helpItem);
		add(optMenu);
		add(infoMenu);
	}
	public void onDelete() {
	    helpItem.setSelected(false);
    }
}
