/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package parser;
import java.util.ArrayList;

import math.Util;
import math.VecMath;

final class OperatorExpression<T extends Number> extends Expression<T> {

	private ArrayList<String> operators;
	OperatorExpression(VecMath<T> mt) {
		super(mt);
		this.operators = new ArrayList<String>();
	}
	T getValue() throws Exception {
		int binOpCnt = operators.size();
		for(String op:operators)
			if(Symbols.isUnary(op))
				binOpCnt--;
		if(binOpCnt+1!=getChildCount())
			throw new Exception("Bad operator expression");
		ArrayList<T> vals = new ArrayList<T>();
		for(int i = 0;i<getChildCount();i++)
			vals.add(getChild(i).getValue());
		ArrayList<String> ops = new ArrayList<String>(operators);
		while(ops.size()>0) {
			int[] opPos = Symbols.precedence(ops);
			String op = ops.remove(opPos[0]);
			T arg1 = vals.remove(opPos[0]);
			T arg2 = null;
			if(!Symbols.isUnary(op))
				arg2 = vals.remove(opPos[0]);
			T res = calculate(op,arg1,arg2);
			vals.add(opPos[0],res);
		}
		return vals.get(0);
	}
	private T calculate(String op,T arg1,T arg2) throws Exception {
		if(op.compareTo(Symbols.MUL)==0)
			return math.mul(arg1,arg2);
		if(op.compareTo(Symbols.DIV)==0)
			return math.div(arg1,arg2);
		if(op.compareTo(Symbols.ADD)==0)
			return math.add(arg1,arg2);
		if(op.compareTo(Symbols.SUB)==0)
			return math.sub(arg1,arg2);
		if(op.compareTo(Symbols.POW)==0) {
			int iarg2 = arg2.intValue();
			if(math.cmp(arg2,math.util.cast(iarg2))==0)
				return math.pow(arg1,iarg2);
			return math.pow(arg1,arg2);
		}
		if(op.compareTo(Symbols.FAC)==0) {
			if(math.cmp(arg1,math.util.round(arg1,1))!=0)
				throw new Exception("Can't calculate the faculty of non-integers");
			if(math.cmp(arg1,math.ZERO)<0)
				throw new Exception("Can't calculate the faculty of negative integers");
			T res = math.ONE;
			for(long i = 2;i<=arg1.longValue();i++)
				res = math.mul(res,math.util.cast(i));
			return math.util.copy(res,Util.MAX_INT);
		}
		throw new Exception("Unknown operator "+op);
	}
	void addOperator(String op) {
		operators.add(op);
	}
}
