/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package parser;
abstract class Parser {

	void parse(String s) throws Exception {
		StringBuffer expression = removeChars(new StringBuffer(s));
		int valIdx = -1;
		int braceCtr = 0;
		main_loop: for(int i = 0;i<expression.length();i++) {
			char pc = i>0?expression.charAt(i-1):0;
			char c = expression.charAt(i);
			for(char op:new char[]{'+','-'})
				if(c==op) {
					if(i==0||pc=='('||pc=='{'||pc=='[')
						number("0");
					else if(pc=='E'||pc=='e')
						continue main_loop;
					else if(Character.isLetterOrDigit(pc)) {
						parse(valIdx,i,expression);
						valIdx = -1;
					}
					operator(op+"");
					continue main_loop;
				}
			for(char op:new char[]{'*','/','^','!'})
				if(c==op) {
					parse(valIdx,i,expression);
					valIdx = -1;
					operator(op+"");
					continue main_loop;
				}
			char[] br = new char[]{'(','[','{',')',']','}'};
			for(byte k = 0;k<br.length;k++)
				if(c==br[k]) {
					parse(valIdx,i,expression);
					valIdx = -1;
					if(k<3) {
						openBrace();
						braceCtr++;
					} else {
						if(--braceCtr<0)
							throw new Exception("Missing opening brace");
						closeBrace();
					}
					continue main_loop;
				}
			if(Character.isLetterOrDigit(c)||c=='.') {
				if(valIdx==-1)
					valIdx = i;
			} else
				throw new Exception("Unknown symbol: "+c);
		}
		if(braceCtr>0)
			throw new Exception("Missing closing brace");
		parse(valIdx,expression.length(),expression);
	}
	private static StringBuffer removeChars(StringBuffer sb) {
		String[] delList = new String[]{" ","\n","\t","\b","\f","\r"};
		for(String str:delList)
			while(sb.indexOf(str)!=-1)
				sb.deleteCharAt(sb.indexOf(str));
		return sb;
	}
	private void parse(int valIdx,int idx,StringBuffer expr) throws Exception {
		if(valIdx!=-1) {
			String s = expr.substring(valIdx,idx);
			if(s==null||s.compareTo("")==0)
				throw new Exception("Error while parsing expression "+expr);
			if(Symbols.isFunction(s))
				function(s);
			else {
				if(Character.isLetter(s.charAt(0)))
					variable(s);
				else if(Character.isDigit(s.charAt(0)))
					try {
						number(s);
					} catch(Exception e) {
						String msg = "Invalid expression: "+s+".\nMaybe a * symbol is missing.";
						throw new Exception(msg);
					}
			}
		}
	}
	abstract void openBrace();
	abstract void closeBrace();
	abstract void operator(String op) throws Exception;
	abstract void variable(String v);
	abstract void function(String f);
	abstract void number(String val) throws Exception;
}
