/**
	Copyright (c) 2009-2012 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package test;
import junit.framework.TestCase;
import lattice.Box2D;
import lattice.Ray;
import math.VecMath;
import math.VecMathApfloat;
import math.VecMathFloat;

import org.apfloat.Apfloat;
import org.junit.Test;

public class Box2DTest {

	private static final float DELTA = 0.000001f;
	@Test
	public void testIntersect2D() {
		Float[] corner1 = new Float[]{-1.0f,-1.0f};
		Float[][] sides = new Float[][]{new Float[]{1.0f,0.0f},new Float[]{0.0f,1.0f}};
		Float[] lengths = new Float[]{2.0f,2.0f};
		VecMathFloat math = new VecMathFloat();
		Box2D<Float> box2d = new Box2D<Float>(corner1,sides,lengths,math);
		intersect2D(box2d,math);
		// Check scaling and moving the corner
		Float[] corner2 = new Float[]{4.0f,5.0f};
		sides = new Float[][]{new Float[]{1.0f,0.0f},new Float[]{0.0f,1.0f}};
		lengths = new Float[]{5.0f,8.0f};
		box2d = new Box2D<Float>(corner2,sides,lengths,math);
		box2d.setLength(2.0f);
		box2d.setCorner(corner1);
		intersect2D(box2d,math);
	}
	private void intersect2D(Box2D<Float> box2d,VecMath<Float> math) {
		// outer intersecting rays
		Float[] start = new Float[]{-1.5f,0.5f};
		Float[] dir = new Float[]{1.0f,0.0f};
		Ray<Float> ray = new Ray<Float>(start,dir,null);
		Float[] isctPoint = new Float[2];
		boolean result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",result,true);
		TestCase.assertEquals("Intersection point x",-1.0f,isctPoint[0],DELTA);
		TestCase.assertEquals("Intersection point y",0.5,isctPoint[1],DELTA);
		start = new Float[]{1.5f,0.7f};
		dir = new Float[]{-1.0f,0.0f};
		ray = new Ray<Float>(start,dir,null);
		isctPoint = new Float[2];
		result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",result,true);
		TestCase.assertEquals("Intersection point x",1.0f,isctPoint[0],DELTA);
		TestCase.assertEquals("Intersection point y",0.7,isctPoint[1],DELTA);
		start = new Float[]{1.0f,2.0f};
		dir = new Float[]{-1.0f,-1.0f};
		math.normalize(dir);
		ray = new Ray<Float>(start,dir,null);
		isctPoint = new Float[2];
		result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",result,true);
		TestCase.assertEquals("Intersection point x",0.0f,isctPoint[0],DELTA);
		TestCase.assertEquals("Intersection point y",1.0f,isctPoint[1],DELTA);
		start = new Float[]{1.0f,3.0f};
		dir = new Float[]{-1.0f,-1.0f};
		math.normalize(dir);
		ray = new Ray<Float>(start,dir,null);
		isctPoint = new Float[2];
		result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",result,true);
		TestCase.assertEquals("Intersection point x",-1.0f,isctPoint[0],DELTA);
		TestCase.assertEquals("Intersection point y",1.0f,isctPoint[1],DELTA);
		start = new Float[]{-1.0f,-2.0f};
		dir = new Float[]{1.0f,1.0f};
		math.normalize(dir);
		ray = new Ray<Float>(start,dir,null);
		isctPoint = new Float[2];
		result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",result,true);
		TestCase.assertEquals("Intersection point x",0.0f,isctPoint[0],DELTA);
		TestCase.assertEquals("Intersection point y",-1.0f,isctPoint[1],DELTA);
		start = new Float[]{-1.5f,0.5f};
		dir = new Float[]{0.9f,-0.1f};
		ray = new Ray<Float>(start,dir,null);
		isctPoint = new Float[2];
		result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",result,true);
		start = new Float[]{1.5f,0.6f};
		dir = new Float[]{-0.9f,-0.1f};
		math.normalize(dir);
		ray = new Ray<Float>(start,dir,null);
		isctPoint = new Float[2];
		result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",result,true);
		start = new Float[]{0.6f,1.5f};
		dir = new Float[]{-0.1f,-0.9f};
		math.normalize(dir);
		ray = new Ray<Float>(start,dir,null);
		isctPoint = new Float[2];
		result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",result,true);
		start = new Float[]{-0.6f,-1.5f};
		dir = new Float[]{0.1f,0.9f};
		math.normalize(dir);
		ray = new Ray<Float>(start,dir,null);
		isctPoint = new Float[2];
		result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",result,true);
		// inner intersecting rays
		start = new Float[]{0.0f,-0.5f};
		dir = new Float[]{1.0f,1.0f};
		math.normalize(dir);
		ray = new Ray<Float>(start,dir,null);
		isctPoint = new Float[2];
		result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",result,true);
		TestCase.assertEquals("Intersection point x",1.0f,isctPoint[0],DELTA);
		TestCase.assertEquals("Intersection point y",0.5,isctPoint[1],DELTA);
		start = new Float[]{0.5f,0.0f};
		dir = new Float[]{-1.5f,0.25f};
		math.normalize(dir);
		ray = new Ray<Float>(start,dir,null);
		isctPoint = new Float[2];
		result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",result,true);
		TestCase.assertEquals("Intersection point x",-1.0f,isctPoint[0],DELTA);
		TestCase.assertEquals("Intersection point y",0.25,isctPoint[1],DELTA);
		// inner intersecting rays starting on bound
		start = new Float[]{-1.0f,0.5f};
		dir = new Float[]{1.0f,1.0f};
		math.normalize(dir);
		ray = new Ray<Float>(start,dir,null);
		isctPoint = new Float[2];
		result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",result,true);
		TestCase.assertEquals("Intersection point x",-1.0f,isctPoint[0],DELTA);
		TestCase.assertEquals("Intersection point y",0.5,isctPoint[1],DELTA);
		math.neg(ray.getDir());
		TestCase.assertEquals("Has intersection",result,true);
		TestCase.assertEquals("Intersection point x",-1.0f,isctPoint[0],DELTA);
		TestCase.assertEquals("Intersection point y",0.5,isctPoint[1],DELTA);
		// outer nonintersecting rays
		start = new Float[]{0.0f,-3.0f};
		dir = new Float[]{1.0f,1.0f};
		math.normalize(dir);
		ray = new Ray<Float>(start,dir,null);
		isctPoint = new Float[2];
		result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",result,false);
		start = new Float[]{0.0f,3.0f};
		dir = new Float[]{-1.0f,-1.0f};
		math.normalize(dir);
		ray = new Ray<Float>(start,dir,null);
		isctPoint = new Float[2];
		result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",result,false);
		start = new Float[]{5.0f,1.0f};
		dir = new Float[]{-0.5f,0.5f};
		math.normalize(dir);
		ray = new Ray<Float>(start,dir,null);
		isctPoint = new Float[2];
		result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",result,false);
		start = new Float[]{-1.5f,0f};
		dir = new Float[]{-1f,0f};
		ray = new Ray<Float>(start,dir,null);
		isctPoint = new Float[2];
		result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",result,false);
		start = new Float[]{1.5f,0f};
		dir = new Float[]{1f,0f};
		ray = new Ray<Float>(start,dir,null);
		isctPoint = new Float[2];
		result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",result,false);
		start = new Float[]{0f,-1.5f};
		dir = new Float[]{0f,-1.5f};
		ray = new Ray<Float>(start,dir,null);
		isctPoint = new Float[2];
		result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",result,false);
		start = new Float[]{0f,1.5f};
		dir = new Float[]{0f,1.5f};
		ray = new Ray<Float>(start,dir,null);
		isctPoint = new Float[2];
		result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",result,false);
	}
	@Test
	public void testIntersect3D() {
		VecMath<Float> math = new VecMathFloat();
		Float[] corner1 = new Float[]{-1.0f,-1.0f,-1.0f};
		Float[][] bases = new Float[3][3];
		bases[0] = new Float[]{1.0f,0.0f,0.0f};
		bases[1] = new Float[]{0.0f,math.sqrt(0.5f),math.sqrt(0.5f)};
		math.cross(bases[0],bases[1],bases[2]);
		Float[] lengths = new Float[]{2.0f,math.sqrt(8.0f)};
		Box2D<Float> box2d = new Box2D<Float>(corner1,bases,lengths,math);
		intersect3D(box2d,math);
		Float[] corner2 = new Float[]{7.0f,9.0f,14.0f};
		bases[0] = new Float[]{1.0f,0.0f,0.0f};
		bases[1] = new Float[]{0.0f,math.sqrt(0.5f),math.sqrt(0.5f)};
		math.cross(bases[0],bases[1],bases[2]);
		lengths = new Float[]{8.0f,math.sqrt(50.0f)};
		box2d = new Box2D<Float>(corner2,bases,lengths,math);
		box2d.setLength(2.0f,(byte)0);
		box2d.setLength((float)Math.sqrt(8),(byte)1);
		box2d.setCorner(corner1);
		intersect3D(box2d,math);
	}
	private void intersect3D(Box2D<Float> box2d,VecMath<Float> math) {
		// outer intersecting rays
		Float[] start = new Float[]{0.0f,0.0f,0.5f};
		Float[] dir = new Float[]{0.0f,1.0f,0.0f};
		Ray<Float> ray = new Ray<Float>(start,dir,null);
		Float[] isctPoint = new Float[3];
		boolean result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",true,result);
		TestCase.assertEquals("Intersection point x",0.0f,isctPoint[0],DELTA);
		TestCase.assertEquals("Intersection point y",0.5,isctPoint[1],DELTA);
		TestCase.assertEquals("Intersection point z",0.5,isctPoint[2],DELTA);
		start = new Float[]{0.5f,0.5f,0.0f};
		dir = new Float[]{0.0f,0.0f,1.0f};
		ray = new Ray<Float>(start,dir,null);
		isctPoint = new Float[3];
		result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",result,true);
		TestCase.assertEquals("Intersection point x",0.5,isctPoint[0],DELTA);
		TestCase.assertEquals("Intersection point y",0.5,isctPoint[1],DELTA);
		TestCase.assertEquals("Intersection point z",0.5,isctPoint[2],DELTA);
		start = new Float[]{2.0f,0.5f,0.5f};
		dir = new Float[]{-1.0f,0.0f,0.0f};
		ray = new Ray<Float>(start,dir,null);
		isctPoint = new Float[3];
		result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",result,true);
		TestCase.assertEquals("Intersection point x",1.0f,isctPoint[0],DELTA);
		TestCase.assertEquals("Intersection point y",0.5,isctPoint[1],DELTA);
		TestCase.assertEquals("Intersection point z",0.5,isctPoint[2],DELTA);
		start = new Float[]{-1.0f,2.0f,2.0f};
		dir = new Float[]{1.0f,-1.0f,-1.0f};
		math.normalize(dir);
		ray = new Ray<Float>(start,dir,null);
		isctPoint = new Float[3];
		result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",result,true);
		TestCase.assertEquals("Intersection point x",0.0f,isctPoint[0],DELTA);
		TestCase.assertEquals("Intersection point y",1.0f,isctPoint[1],DELTA);
		TestCase.assertEquals("Intersection point z",1.0f,isctPoint[2],DELTA);
		// intersecting ray starting on bound
		start = new Float[]{0.5f,-1.0f,-1.0f};
		dir = new Float[]{0.0f,1.0f,1.0f};
		math.normalize(dir);
		ray = new Ray<Float>(start,dir,null);
		isctPoint = new Float[3];
		result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",result,true);
		TestCase.assertEquals("Intersection point x",0.5,isctPoint[0],DELTA);
		TestCase.assertEquals("Intersection point y",-1.0f,isctPoint[1],DELTA);
		TestCase.assertEquals("Intersection point z",-1.0f,isctPoint[2],DELTA);
		math.neg(ray.getDir());
		isctPoint = new Float[3];
		result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",result,true);
		TestCase.assertEquals("Intersection point x",0.5,isctPoint[0],DELTA);
		TestCase.assertEquals("Intersection point y",-1.0f,isctPoint[1],DELTA);
		TestCase.assertEquals("Intersection point z",-1.0f,isctPoint[2],DELTA);
		// non intersecting rays
		start = new Float[]{-1.0f,2.01f,2.0f};
		dir = new Float[]{1.0f,-1.0f,-1.0f};
		math.normalize(dir);
		ray = new Ray<Float>(start,dir,null);
		isctPoint = new Float[3];
		result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",result,false);
		start = new Float[]{-1.0f,1.99f,2.0f};
		dir = new Float[]{1.0f,-1.0f,-1.0f};
		math.normalize(dir);
		ray = new Ray<Float>(start,dir,null);
		isctPoint = new Float[3];
		result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",result,false);
		start = new Float[]{0.0f,-0.5f,0.5f};
		dir = new Float[]{0.0f,-1.0f,1.0f};
		math.normalize(dir);
		ray = new Ray<Float>(start,dir,null);
		isctPoint = new Float[3];
		result = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",result,false);
	}
	@Test
	public void testPreciseIntersect2D() {
		VecMathApfloat math = new VecMathApfloat((short)100);
		Apfloat[] corner1 = new Apfloat[]{math.MINUS_ONE,math.MINUS_ONE};
		Apfloat[][] sides = new Apfloat[2][2];
		sides[0] = new Apfloat[]{math.ONE,math.ZERO};
		sides[1] = new Apfloat[]{math.ZERO,math.ONE};
		Apfloat[] lengths = new Apfloat[]{math.TWO,math.TWO};
		Box2D<Apfloat> box2d = new Box2D<Apfloat>(corner1,sides,lengths,math);
		box2d.rotate(math.ZERO2D,null,math.neg(math.PI_OVER_FOUR));
		Apfloat[] start = new Apfloat[]{math.THREE.divide(math.TWO),math.ZERO};
		Apfloat[] dir = new Apfloat[]{math.MINUS_ONE,math.ZERO};
		Ray<Apfloat> ray = new Ray<Apfloat>(start,dir,null);
		Apfloat[] isctPoint = new Apfloat[2];
		boolean b = box2d.intersect(ray,isctPoint);
		TestCase.assertEquals("Has intersection",true,b);
		Apfloat epsilon = math.mul(math.EPSILON,math.util.cast(100));
		assertEquals("isct x",math.sqrt(math.util.copy(math.TWO,100)),isctPoint[0],epsilon,math);
		assertEquals("isct y",math.ZERO,isctPoint[1],epsilon,math);
		start = new Apfloat[]{math.neg(math.TWO),math.ZERO};
		dir = new Apfloat[]{math.ONE,math.ZERO};
		ray = new Ray<Apfloat>(start,dir,null);
		isctPoint = new Apfloat[2];
		box2d.intersect(ray,isctPoint);
		assertEquals("isct x",math.neg(math.sqrt(math.util.copy(math.TWO,100))),isctPoint[0],
		        epsilon,math);
		assertEquals("isct y",math.ZERO,isctPoint[1],epsilon,math);
		start = new Apfloat[]{math.ZERO,math.TWO};
		dir = new Apfloat[]{math.ZERO,math.MINUS_ONE};
		ray = new Ray<Apfloat>(start,dir,null);
		isctPoint = new Apfloat[2];
		box2d.intersect(ray,isctPoint);
		assertEquals("isct x",math.ZERO,isctPoint[0],epsilon,math);
		assertEquals("isct y",math.sqrt(math.util.copy(math.TWO,100)),isctPoint[1],epsilon,math);
		start = new Apfloat[]{math.ZERO,math.neg(math.TWO)};
		dir = new Apfloat[]{math.ZERO,math.ONE};
		ray = new Ray<Apfloat>(start,dir,null);
		isctPoint = new Apfloat[2];
		box2d.intersect(ray,isctPoint);
		assertEquals("isct x",math.ZERO,isctPoint[0],epsilon,math);
		assertEquals("isct y",math.neg(math.sqrt(math.util.copy(math.TWO,100))),isctPoint[1],
		        epsilon,math);
	}
	@Test
	public void testContainsLocal() {
		VecMathFloat math = new VecMathFloat();
		Float[] corner1 = new Float[]{-1.0f,-1.0f};
		Float[][] sides = new Float[][]{new Float[]{1.0f,0.0f},new Float[]{0.0f,1.0f}};
		Float[] lengths = new Float[]{2.0f,2.0f};
		Box2D<Float> box = new Box2D<Float>(corner1,sides,lengths,math);
		box.rotate(math.ZERO2D,null,math.PI_OVER_FOUR);
		Float[] point = new Float[]{-1.0f,1.0f};
		Float[] ret = new Float[2];
		box.world2LocalPoint(point,ret);
		boolean result = box.containsLocal(ret);
		TestCase.assertEquals("Contains local",result,false);
		point = new Float[]{-math.sqrt(2.0001f),0.0f};
		ret = new Float[2];
		box.world2LocalPoint(point,ret);
		result = box.containsLocal(ret);
		TestCase.assertEquals("Contains local",result,false);
		point = new Float[]{-math.sqrt(1.9999f),0.0f};
		ret = new Float[2];
		box.world2LocalPoint(point,ret);
		result = box.containsLocal(ret);
		TestCase.assertEquals("Contains local",result,true);
	}
	private <T extends Number>void assertEquals(String msg,T val,T res,T diff,VecMath<T> m) {
		if(m.cmp(m.abs(m.sub(val,res)),diff)>0) {
			System.out.println("Test failed");
			System.out.println(msg);
			System.out.println("Expected: "+val);
			System.out.println("Got: "+res);
			System.out.println("Difference: "+m.sub(val,res));
			System.out.println("Max Difference: "+diff);
			Assert.assertEquals(msg,true,false);
			new Exception("Error location: ").printStackTrace();
			System.exit(0);
		}
	}
}
