/**
	Copyright (c) 2009-2012 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package test;
import math.IFunction;
import math.VecMath;
import math.VecMathApfloat;

import org.apfloat.Apfloat;
import org.apfloat.Apint;
import org.junit.Test;

public class VecMathTest {

	@Test
	public void testSinCosPow10Table() {
		VecMathApfloat math = new VecMathApfloat((short)20);
		Apfloat epsilon = math.EPSILON.multiply(math.TEN);
		for(int ran = 0;ran<math.sinTab.length;ran++) {
			System.out.println("sine table test: idx="+ran);
			// int ran = (int)(Math.random()*math.sinTab.length);
			// int ran = 11;
			Apfloat pifact = math.div(math.util.cast(ran),math.util.cast(math.sinTab.length));
			Apfloat phi = math.mul(math.PI,pifact);
			Apfloat[] sincosTab = math.discrSinCos(pifact.floatValue());
			Apfloat sin = math.sin(phi);
			// System.out.println("pifact: "+pifact);
			// System.out.println("sin:    "+sin);
			// System.out.println("sinTab: "+sincosTab[0]);
			Apfloat dif = math.abs(math.sub(sin,sincosTab[0]));
			// System.out.println("diff: "+dif);
			Assert.assertTrue(math.cmp(dif,epsilon)<=0);
			Apfloat cos = math.cos(phi);
			// System.out.println("cos:    "+cos);
			// System.out.println("cosTab: "+sincosTab[1]);
			dif = math.abs(math.sub(cos,sincosTab[1]));
			// System.out.println("diff:   "+dif);
			Assert.assertTrue(math.cmp(dif,epsilon)<=0);
			// System.out.println(dif);
		}
	}
	@Test
	public void testFindRoot() {
		final VecMathApfloat math = new VecMathApfloat((short)100);
		Apfloat two_p25 = math.util.copy(math.TWO,25);
		IFunction<Apfloat> f = new IFunction<Apfloat>() {

			@Override
			public Apfloat f(Apfloat x) {
				return math.sub(math.TWO,math.mul(x,x));
			}
		};
		IFunction<Apfloat> dfOverDx = new IFunction<Apfloat>() {

			@Override
			public Apfloat f(Apfloat x) {
				return math.mul(x,-2);
			}
		};
		Apfloat root = null;
		try {
			root = math.findRoot(f,dfOverDx,two_p25);
		} catch(Exception e) {
			e.printStackTrace();
		}
		System.out.println("root1 = "+root);
		System.out.println("root2 = "+math.sqrt(two_p25));
	}
	@Test
	public void testScale() {
		final VecMathApfloat math = new VecMathApfloat((short)100);
		Apfloat x = new Apfloat("0.0123456789");
		int s = math.util.getScale(x);
		Assert.assertEquals("Scale",-2,s);
		x = new Apfloat("0.123456789");
		s = math.util.getScale(x);
		Assert.assertEquals("Scale",-1,s);
		x = new Apfloat("1.23456789");
		s = math.util.getScale(x);
		Assert.assertEquals("Scale",0,s);
		x = new Apfloat("12.3456789");
		s = math.util.getScale(x);
		Assert.assertEquals("Scale",1,s);
	}
	@Test
	public void testRound() {
		final VecMathApfloat math = new VecMathApfloat((short)100);
		Apfloat x = new Apfloat("1.23456789");
		Apfloat y = math.util.round(x,6);
		Assert.assertTrue("Round",math.cmp(math.sub(new Apfloat("1.234568"),y),math.ZERO)==0);
		y = math.util.round(x,4);
		Assert.assertTrue("Round",math.cmp(math.sub(new Apfloat("1.2346"),y),math.ZERO)==0);
		y = math.util.round(x,3);
		Assert.assertTrue("Round",math.cmp(math.sub(new Apfloat("1.235"),y),math.ZERO)==0);
		y = math.util.round(x,2);
		Assert.assertTrue("Round",math.cmp(math.sub(new Apfloat("1.23"),y),math.ZERO)==0);
		y = math.util.round(x,15);
		Assert.assertTrue("Round",math.cmp(math.sub(x,y),math.ZERO)==0);
		y = math.util.round(x,30);
		Assert.assertTrue("Round",math.cmp(math.sub(x,y),math.ZERO)==0);
		x = new Apfloat("1.23456789122345678912234567891223456789122345678912");
		y = math.util.round(x,48);
		Apfloat res = new Apfloat("1.234567891223456789122345678912234567891223456789");
		Assert.assertTrue("Round",math.cmp(math.sub(res,y),math.ZERO)==0);
		y = math.util.round(x,47);
		res = new Apfloat("1.23456789122345678912234567891223456789122345679");
		Assert.assertTrue("Round",math.cmp(math.sub(res,y),math.ZERO)==0);
		x = new Apfloat("0.0078941");
		y = math.util.round(x,4);
		res = new Apfloat("0.0079");
		Assert.assertTrue("Round",math.cmp(math.sub(res,y),math.ZERO)==0);
		x = new Apfloat("0.0078941");
		y = math.util.round(x,8);
		res = x;
		Assert.assertTrue("Round",math.cmp(math.sub(res,y),math.ZERO)==0);
		x = new Apfloat("0.0078941");
		y = math.util.round(x,7);
		res = new Apfloat("0.007894");
		Assert.assertTrue("Round",math.cmp(math.sub(res,y),math.ZERO)==0);
		x = new Apfloat("0.007899999999999999999999999999999999999999999999");
		y = math.util.round(x,10);
		res = new Apfloat("0.0079");
		Assert.assertTrue("Round",math.cmp(math.sub(res,y),math.ZERO)==0);
		y = math.util.round(x,Util.MAX_INT);
		res = x;
		Assert.assertTrue("Round",math.cmp(math.sub(res,y),math.ZERO)==0);
		x = new Apfloat("789999999999.9999999999999999999999999999999999");
		y = math.util.round(x,Util.MAX_INT);
		res = x;
		Assert.assertTrue("Round",math.cmp(math.sub(res,y),math.ZERO)==0);
	}
	@Test
	public void testErrorScale() {
		final VecMathApfloat math = new VecMathApfloat((short)100);
		Apfloat x = new Apfloat("12345678.9");
		int es = math.util.getErrorScale(x);
		Assert.assertEquals("Error scale",-1,es);
		x = new Apfloat("1.23456789");
		es = math.util.getErrorScale(x);
		Assert.assertEquals("Error scale",-8,es);
		x = new Apfloat("0.234567");
		es = math.util.getErrorScale(x);
		Assert.assertEquals("Error scale",-6,es);
		x = new Apfloat("0.0000000123456789");
		es = math.util.getErrorScale(x);
		Assert.assertEquals("Error scale",-16,es);
		x = new Apfloat("0.00532e-15");
		es = math.util.getErrorScale(x);
		Assert.assertEquals("Error scale",-20,es);
		x = new Apint("24");
		es = math.util.getErrorScale(x);
		Assert.assertEquals("Error scale",Integer.MIN_VALUE+1,es);
		x = x.precision(4);
		es = math.util.getErrorScale(x);
		Assert.assertEquals("Error scale",-2,es);
		x = new Apfloat("0.0053200");
		es = math.util.getErrorScale(x);
		Assert.assertEquals("Error scale",-7,es);
	}
	@Test
	public void testSinCos() {
		final VecMathApfloat m = new VecMathApfloat((short)50);
		Apfloat phi0 = m.mul(m.PI,m.TEN);
		Apfloat eps = m.EPSILON.multiply(m.HUNDRED);
		for(Apfloat phi = phi0;m.cmp(phi,m.neg(phi0))>=0;phi = m.sub(phi,m.PI_OVER_EIGHT)) {
			Apfloat[] sc = m.sincos(phi);
			//System.out.println(phi);		
			assertEquals("sincos[0]",sc[0],m.sin(phi),eps,m);
			assertEquals("sincos[1]",sc[1],m.cos(phi),eps,m);
		}
	}
	@Test 
	public void testDiscrSinCos(){
		final VecMathApfloat m = new VecMathApfloat((short)20);
		Apfloat eps = m.EPSILON.multiply(m.HUNDRED);
		for(float pifactor = 0.9999f;pifactor>=-0.9999;pifactor-=1f/3333){
			Apfloat[] discrSinCos = m.discrSinCos(pifactor);
			Apfloat discrPhi = m.discrAngle(pifactor);
			Apfloat[] sinCos = m.sincos(discrPhi);
			//System.out.println(pifactor);
			assertEquals("sincos[0]",discrSinCos[0],sinCos[0],eps,m);
			assertEquals("sincos[1]",discrSinCos[1],sinCos[1],eps,m);
		}
	}
	private <T extends Number>void assertEquals(String msg,T val,T res,T diff,VecMath<T> m) {
		if(m.cmp(m.abs(m.sub(val,res)),diff)>0) {
			System.out.println("Test failed");
			System.out.println(msg);
			System.out.println("Expected: "+val);
			System.out.println("Got: "+res);
			System.out.println("Difference: "+m.sub(val,res));
			System.out.println("Max Difference: "+diff);
			Assert.assertEquals(msg,true,false);
			System.exit(0);
		}
	}
}
